@@ -1,11 +1,12 @@
 """Events from pubsub subscriber."""
 
-from abc import abstractmethod, ABC
 import datetime
+import logging
+from abc import ABC, abstractmethod
 
 from .auth import AbstractAuth
-from .traits import BuildTraits, Command
 from .registry import Registry
+from .traits import BuildTraits, Command
 
 EVENT_ID = "eventId"
 EVENT_SESSION_ID = "eventSessionId"
@@ -19,48 +20,74 @@ TYPE = "type"
 SUBJECT = "subject"
 OBJECT = "object"
 
+# Event images expire 30 seconds after the event is published
+EVENT_IMAGE_EXPIRE_SECS = 30
+
 EVENT_MAP = Registry()
 
+_LOGGER = logging.getLogger(__name__)
 
-class EventBase(ABC):
-    """Base class for all event types."""
 
-    def __init__(self, data):
+class EventProcessingError(Exception):
+    """Raised when there was an error handling an event."""
+
+
+class ImageEventBase(ABC):
+    """Base class for all image related event types."""
+
+    def __init__(self, data: dict, timestamp: datetime.datetime):
         """Initialize EventBase."""
         self._data = data
+        self._timestamp = timestamp
 
     @property
     def event_id(self) -> str:
-        """A unique event identifier."""
+        """An ID associated with the event.
+
+        Can be used with CameraEventImageTrait to download the imaage.
+        """
         return self._data[EVENT_ID]
 
     @property
     def event_session_id(self) -> str:
-        """Used to authenticate follow up request related to this event."""
+        """An ID used to associate separate messages with a single event."""
         return self._data[EVENT_SESSION_ID]
 
+    @property
+    def timestamp(self) -> datetime.datetime:
+        """Timestap when the event occurred."""
+        return self._timestamp
+
+    @property
+    def expires_at(self) -> datetime.datetime:
+        return self._timestamp + datetime.timedelta(seconds=EVENT_IMAGE_EXPIRE_SECS)
+
 
 @EVENT_MAP.register()
-class CameraMotionEvent(EventBase):
+class CameraMotionEvent(ImageEventBase):
     """Motion has been detected by the camera."""
+
     NAME = "sdm.devices.events.CameraMotion.Motion"
 
 
 @EVENT_MAP.register()
-class CameraPersonEvent(EventBase):
+class CameraPersonEvent(ImageEventBase):
     """A person has been detected by the camera."""
+
     NAME = "sdm.devices.events.CameraPerson.Person"
 
 
 @EVENT_MAP.register()
-class CameraSoundEvent(EventBase):
+class CameraSoundEvent(ImageEventBase):
     """Sound has been detected by the camera."""
+
     NAME = "sdm.devices.events.CameraSound.Sound"
 
 
 @EVENT_MAP.register()
-class DoorbellChimeEvent(EventBase):
+class DoorbellChimeEvent(ImageEventBase):
     """The doorbell has been pressed."""
+
     NAME = "sdm.devices.events.DoorbellChime.Chime"
 
 
@@ -86,14 +113,14 @@ class RelationUpdate:
         return self._raw_data[OBJECT]
 
 
-def BuildEvents(events: dict, event_map: dict) -> dict:
+def BuildEvents(events: dict, event_map: dict, timestamp: datetime.datetime) -> dict:
     """Builds a trait map out of a response dict."""
     result = {}
     for (event, event_data) in events.items():
-        if not event in event_map:
+        if event not in event_map:
             continue
         cls = event_map[event]
-        result[event] = cls(event_data)
+        result[event] = cls(event_data, timestamp)
     return result
 
 
@@ -102,13 +129,14 @@ class EventMessage:
 
     def __init__(self, raw_data: dict, auth: AbstractAuth):
         """Initialize an EventMessage."""
+        _LOGGER.debug("EventMessage raw_data=%s", raw_data)
         self._raw_data = raw_data
         self._auth = auth
 
     @property
     def event_id(self) -> str:
         """A unique event identifier."""
-        return self._raw_data[EVENT_ID]
+        return self._raw_data.get(EVENT_ID, None)
 
     @property
     def timestamp(self) -> datetime.datetime:
@@ -119,22 +147,22 @@ class EventMessage:
     @property
     def resource_update_name(self) -> str:
         """Returns the id of the device that was updated."""
-        if not RESOURCE_UPDATE in self._raw_data:
+        if RESOURCE_UPDATE not in self._raw_data:
             return None
         return self._raw_data[RESOURCE_UPDATE][NAME]
 
     @property
     def resource_update_events(self) -> dict:
         """Returns the set of events that happened."""
-        if not RESOURCE_UPDATE in self._raw_data:
+        if RESOURCE_UPDATE not in self._raw_data:
             return None
         events = self._raw_data[RESOURCE_UPDATE].get(EVENTS, {})
-        return BuildEvents(events, EVENT_MAP)
+        return BuildEvents(events, EVENT_MAP, self.timestamp)
 
     @property
     def resource_update_traits(self) -> dict:
         """Returns the set of traits that were updated."""
-        if not RESOURCE_UPDATE in self._raw_data:
+        if RESOURCE_UPDATE not in self._raw_data:
             return None
         cmd = Command(self.resource_update_name, self._auth)
         events = self._raw_data[RESOURCE_UPDATE].get(TRAITS, {})
@@ -143,7 +171,7 @@ class EventMessage:
     @property
     def relation_update(self) -> RelationUpdate:
         """Represent a relational update for a resource."""
-        if not RELATION_UPDATE in self._raw_data:
+        if RELATION_UPDATE not in self._raw_data:
             return None
         return RelationUpdate(self._raw_data[RELATION_UPDATE])
 
